searchNodes=[{"type":"module","title":"Corsica","doc":"Plug -based swiss-army knife for CORS requests. Corsica provides facilities for dealing with CORS requests and responses. It provides: low-level functions that let you decide when and where to deal with CORS requests and CORS response headers; a plug that handles CORS requests and responds to preflight requests; a router that can be used in your modules in order to turn them into CORS handlers which provide fine control over dealing with CORS requests. How It Works Corsica is compliant with the W3C CORS specification . As per this specification, Corsica doesn't put any CORS response headers in a connection that holds an invalid CORS request. To know what &quot;invalid&quot; CORS request means, have a look at the Validity of CORS Requests section below. Headers or No Headers? When some options that are not mandatory and have no default value (such as :max_age ) are not passed to Corsica, the relative header will often not be sent at all. This is compliant with the specification, and at the same time it reduces the size of the response, even if just by a handful of bytes. The following is a list of all the CORS response headers supported by Corsica: Access-Control-Allow-Origin Access-Control-Allow-Methods Access-Control-Allow-Headers Access-Control-Allow-Credentials Access-Control-Allow-Private-Network Access-Control-Expose-Headers Access-Control-Max-Age Vary (see the relevant section below) Options Corsica supports the following options, in the use macro, in Corsica.Router.resource/2 , and in the Corsica plug. :origins ( origin/0 , list of origin/0 , or the string &quot;*&quot; ) This option is required . The origin of a request (specified by the &quot;origin&quot; request header) will be considered a valid origin if it &quot;matches&quot; at least one of the origins specified in :origins . What &quot;matches&quot; means depends on the type of origin. See origin/0 for more information. The value &quot;*&quot; can also be used to match every origin and reply with * as the value of the access-control-allow-origin header. If &quot;*&quot; is used, it must be used as the only value of :origins (that is, it can't be used inside a list of accepted origins). For example: # Matches everything. plug Corsica , origins : &quot;*&quot; # Matches one of the given origins plug Corsica , origins : [ &quot;http://foo.com&quot; , &quot;http://bar.com&quot; ] # Matches the given regex plug Corsica , origins : ~r{^https?://(.*.?)foo.com$} The Origin Showed to Clients This option directly influences the value of the access-control-allow-origin response header. When :origins is &quot;*&quot; , the access-control-allow-origin header is set to * as well. If the request's origin is allowed and :origins is something different than &quot;*&quot; , then you won't see that value as the value of the access-control-allow-origin header: the value of this header will be the request's origin (which is mirrored ). This behaviour is intentional: it's compliant with the W3C CORS specification and at the same time it provides the advantage of &quot;hiding&quot; all the allowed origins from the client (which only sees its origin as an allowed origin). :allow_methods (list of String.t/0 , or :all ) - This is the list of methods allowed in the access-control-request-method header of preflight requests. If the method requested by the preflight request is in this list or is a simple method ( HEAD , GET , or POST ), then that method is always allowed. The methods specified by this option are returned in the access-control-allow-methods response header. If the value of this option is :all , all request methods are allowed and only the method in access-control-request-method is returned as the value of the access-control-allow-methods header. Defaults to [&quot;PUT&quot;, &quot;PATCH&quot;, &quot;DELETE&quot;] (which means these methods are allowed alongside simple methods ). :allow_headers (list of String.t/0 , or :all ) - This is the list of headers allowed in the access-control-request-headers header of preflight requests. If a header requested by the preflight request is in this list or is a simple header , then that header is always allowed. These are the simple headers defined in the spec: Accept Accept-Language Content-Language The headers specified by this option are returned in the access-control-allow-headers response header. If the value of this option is :all , all request headers are allowed and only the headers in access-control-request-headers are returned as the value of the access-control-allow-headers header. Defaults to [] (which means only the simple headers are allowed) :allow_credentials ( boolean/0 ) - If true , sends the access-control-allow-credentials with value true . If false , prevents that header from being sent at all. Defaults to false . Access-Control-Allow-Origin Header with Credentials If :origins is set to &quot;*&quot; and :allow_credentials is set to true , then the value of the access-control-allow-origin header will always be the value of the origin request header (as per the W3C CORS specification) and not * . :allow_private_network ( boolean/0 0 - If true , sets the value of the access-control-allow-private-network header used with preflight requests, which indicates that a resource can be safely shared with external networks. If false , the access-control-allow-private-network is not sent at all. Defaults to false . :expose_headers (list of String.t/0 ) Sets the value of the access-control-expose-headers response header. This option does not have a default value; if it's not provided, the access-control-expose-headers header is not sent at all. :max_age ( String.t/0 or non_neg_integer/0 ) Sets the value of the access-control-max-age header used with preflight requests. This option does not have a default value; if it's not provided, the access-control-max-age header is not sent at all. :telemetry_metadata ( map/0 ) - extra telemetry metadata to be included in all emitted events. This can be useful for identifying which plug Corsica call is emitting the events. See Corsica.Telemetry for more information on Telemetry in Corsica. Available since v2.0.0. :passthrough_non_cors_requests ( boolean/0 ) - If true , allows non-CORS requests to pass through the plug. See cors_req?/1 and preflight_req?/1 to understand what constitutes a CORS request. What we mean by &quot;allowing non-CORS requests&quot; means that Corsica won't verify the Origin header and such, but will still add CORS headers to the response. Defaults to false . Available since v2.1.0. To recap which headers are sent based on options, here's a handy table: Header Request Type Presence in the Response access-control-allow-origin simple, preflight always access-control-allow-headers preflight always access-control-allow-credentials preflight allow_credentials: true access-control-allow-private-network preflight allow_private_network: true access-control-expose-headers preflight :expose_headers is not empty access-control-max-age preflight :max_age is present Usage You can use Corsica as a plug or as a router. Using Corsica as a Plug When Corsica is used as a plug, it intercepts all requests . It only sets a bunch of CORS headers for regular CORS requests, but it responds (with a 200 OK and the appropriate headers) to preflight requests. If you want to use Corsica as a plug, be sure to plug it in your plug pipeline before any router-like plug: routers like Plug.Router (or Phoenix.Router ) respond to HTTP verbs as well as request URLs, so if Corsica is plugged after a router then preflight requests (which are OPTIONS requests), that will often result in 404 errors since no route responds to them. Router-like plugs also include plugs like Plug.Static , which respond to requests and halt the pipeline. defmodule MyApp.Endpoint do plug Head plug Corsica , max_age : 600 , origins : &quot;*&quot; , expose_headers : ~w(X-Foo) plug Plug.Static plug MyApp.Router end Using Corsica as a Router Generator When Corsica is used as a plug, it doesn't provide control over which urls are CORS-enabled or with which options. In order to do that, you can use Corsica.Router . See the documentation for Corsica.Router for more information. The vary Header When Corsica is configured such that the access-control-allow-origin response header will vary depending on the origin request header, then a vary: origin response header will be set. Responding to Preflight Requests When the request is a preflight request and a valid one (valid origin, valid request method, and valid request headers), Corsica directly sends a response to that request instead of just adding headers to the connection (so that a possible plug pipeline can continue). To do this, Corsica halts the connection (through Plug.Conn.halt/1 ) and sends a response . Validity of CORS Requests &quot;Invalid CORS request&quot; can mean that a request doesn't have an Origin header (so it's not a CORS request at all) or that it's a CORS request but: the Origin request header doesn't match any of the allowed origins the request is a preflight request but it requests to use a method or some headers that are not allowed (via the Access-Control-Request-Method and Access-Control-Request-Headers headers) Telemetry Corsica emits some telemetry events. See Corsica.Telemetry for documentation. Logging Corsica used to support Logger logging through the :log option. This option has been removed in v2.0.0 in favor of Telemetry events. If you want to keep the logging behavior, see Corsica.Telemetry.attach_default_handler/1 .","ref":"Corsica.html"},{"type":"function","title":"Corsica.cors_req?/1","doc":"Checks whether a given connection holds a CORS request. This function doesn't check if the CORS request is a valid CORS request: it just checks that it's a CORS request, that is, it has an Origin request header.","ref":"Corsica.html#cors_req?/1"},{"type":"function","title":"Corsica.preflight_req?/1","doc":"Checks whether a given connection holds a preflight CORS request. This function doesn't check that the preflight request is a valid CORS request: it just checks that it's a preflight request. A request is considered to be a CORS preflight request if and only if its request method is OPTIONS and it has a Access-Control-Request-Method request header. Note that if a request is a valid preflight request, that makes it a valid CORS request as well. You can thus call just preflight_req?/1 instead of preflight_req?/1 and cors_req?/1 .","ref":"Corsica.html#preflight_req?/1"},{"type":"function","title":"Corsica.put_cors_preflight_resp_headers/2","doc":"Adds CORS response headers to a preflight request to conn . This function assumes nothing about conn . If conn holds an invalid CORS request or an invalid preflight request, then conn is returned unchanged; the absence of CORS headers will be interpreted as an invalid CORS response by the browser (according to the W3C spec). If the request is a valid CORS request, the following headers will be added to the response: Access-Control-Allow-Origin Access-Control-Allow-Methods Access-Control-Allow-Headers and the following headers will optionally be added (based on the value of the corresponding options): Access-Control-Allow-Credentials (if the :allow_credentials option is true ) Access-Control-Allow-Private-Network (if the :allow_private_network option is true ) Access-Control-Max-Age (if the :max_age option is present) Options This function accepts the same options accepted by the Corsica plug (described in the documentation for the Corsica module). Examples put_cors_preflight_resp_headers conn , [ max_age : 86400 , allow_headers : ~w(X-Header) , allow_private_network : true , origins : ~r/w+.foo.com$/ ]","ref":"Corsica.html#put_cors_preflight_resp_headers/2"},{"type":"function","title":"Corsica.put_cors_simple_resp_headers/2","doc":"Adds CORS response headers to a simple CORS request to conn . This function assumes nothing about conn . If conn holds an invalid CORS request or a request whose origin is not allowed, conn is returned unchanged; the absence of CORS headers will be interpreted as an invalid CORS response by the browser (according to the W3C spec). If the CORS request is valid, the following response headers are set: Access-Control-Allow-Origin and the following headers are optionally set (if the corresponding option is present): Access-Control-Expose-Headers (if the :expose_headers option is present) Access-Control-Allow-Credentials (if the :allow_credentials option is true ) Options This function accepts the same options accepted by the Corsica plug (described in the documentation for the Corsica module). Examples conn |&gt; put_cors_simple_resp_headers ( origins : &quot;*&quot; , allow_credentials : true ) |&gt; send_resp ( 200 , &quot;Hello!&quot; )","ref":"Corsica.html#put_cors_simple_resp_headers/2"},{"type":"function","title":"Corsica.send_preflight_resp/4","doc":"Sends a CORS preflight response regardless of the request being a valid CORS request or not. This function assumes nothing about conn . If it's a valid CORS preflight request with an allowed origin, CORS headers are set by calling put_cors_preflight_resp_headers/2 and the response is sent with status status and body body . conn is halted before being sent. The response is always sent because if the request is not a valid CORS request, then no CORS headers will be added to the response. This behaviour will be interpreted by the browser as a non-allowed preflight request, as expected. For more information on what headers are sent with the response if the preflight request is valid, look at the documentation for put_cors_preflight_resp_headers/2 . Options This function accepts the same options accepted by the Corsica plug (described in the documentation for the Corsica module). Examples This function could be used to manually build a plug that responds to preflight requests. For example: defmodule MyRouter do use Plug.Router plug :match plug :dispatch options &quot;/foo&quot; , do : Corsica . send_preflight_resp ( conn , origins : &quot;*&quot; ) get &quot;/foo&quot; , do : send_resp ( conn , 200 , &quot;ok&quot; ) end","ref":"Corsica.html#send_preflight_resp/4"},{"type":"type","title":"Corsica.options/0","doc":"Options accepted by most functions as well as the Corsica plug.","ref":"Corsica.html#t:options/0"},{"type":"type","title":"Corsica.origin/0","doc":"An origin that can be specified in the :origins option. This is how each type of origin is used in order to check for &quot;matching&quot; origins: strings - the actual origin and the allowed origin have to be identical regexes - the actual origin has to match the allowed regex (as per Regex.match?/2 ) {module, function, args} tuples - module.function is called with two extra arguments prepended to the given args : the current connection and the actual origin; if it returns true the origin is accepted, if it returns false the origin is not accepted.","ref":"Corsica.html#t:origin/0"},{"type":"opaque","title":"Corsica.sanitized_options/0","doc":"Sanitized options, internal to Corsica.","ref":"Corsica.html#t:sanitized_options/0"},{"type":"module","title":"Corsica.Router","doc":"A router to handle and respond to CORS requests. This module provides facilities for creating Plug.Router -based routers that handle CORS requests. A generated router will handle a CORS request by: responding to it if it's a preflight request (refer to Corsica.send_preflight_resp/4 for more information) or adding the right CORS headers to the Plug.Conn connection if it's a valid CORS request. When a module calls use Corsica.Router , it can pass the same options that can be passed to the Corsica plug to the use call. Look at the documentation for the Corsica module for more information about these options. CORS rules in a Corsica.Router can be defined through the resource/2 macro. use Corsica.Router When you use Corsica.Router , your module will become a Plug.Router . Examples defmodule MyApp.CORS do use Corsica.Router , origins : [ &quot;http://foo.com&quot; , &quot;http://bar.com&quot; ] , allow_credentials : true , max_age : 600 resource &quot;/*&quot; # We can override single settings as well. resource &quot;/public/*&quot; , allow_credentials : false end Now in your application's endpoint: defmodule MyApp.Endpoint do plug Plug.Head plug MyApp.CORS plug Plug.Static plug MyApp.Router end Note that a Corsica.Router router will always define a match-all route after the resource routes; this match-all route will simply return the connection unchanged, effectively continuing with the plug pipeline.","ref":"Corsica.Router.html"},{"type":"macro","title":"Corsica.Router.resource/2","doc":"Defines a CORS-enabled resource. This macro takes advantage of the macros defined by Plug.Router (like options/3 and match/3 ) in order to define regular Plug.Router -like routes that efficiently match on the request url; the bodies of the autogenerated routes just perform a couple of checks before calling either Corsica.put_cors_simple_resp_headers/2 or Corsica.send_preflight_resp/4 . Note that if the request is a CORS preflight request (whether it's a valid one or not), a response is immediately sent to the client (whether the request is a valid one or not). This behaviour, combined with the definition of an additional OPTIONS route to route , makes Corsica.Router ideal to just put before any router in a plug pipeline, letting it handle preflight requests by itself. The options given to resource/2 are merged with the default options like it happens with the rest of the functions in the Corsica module. The resource/2 macro also accepts the following options (similar to Plug.Router.match/3 ): :host - the host which the route should match. Defaults to nil , meaning no host match, but can be a string like &quot;example.com&quot; or a string ending with . , like &quot;subdomain.&quot; , for a subdomain match. Examples resource &quot;/foo&quot; , origins : &quot;*&quot; resource &quot;/wildcards/are/ok/*&quot; , max_age : 600 resource &quot;/only/on/subdomain&quot; , host : &quot;mysubdomain.&quot;","ref":"Corsica.Router.html#resource/2"},{"type":"module","title":"Corsica.Telemetry","doc":"Telemetry helpers and documentation around Corsica events. Corsica emits the following events: [:corsica, :accepted_request] — when a CORS request is accepted. [:corsica, :invalid_request] — when a request is not a CORS request. [:corsica, :rejected_request] — when a CORS request is rejected. Includes the following extra metadata: :reason — the reason why the request was rejected. Can be one of: :origin_not_allowed {:req_method_not_allowed, req_method} {:req_headers_not_allowed, non_allowed_headers} Metadata : all the events include the following metadata in addition to any metadata explicitly specified in the list above. :request_type —  :simple or :preflight . :conn - the Plug.Conn struct for the request. Measurements : none of the events include any measurements. Logging Corsica supports basic logging functionality through attach_default_handler/0 .","ref":"Corsica.Telemetry.html"},{"type":"function","title":"Corsica.Telemetry.attach_default_handler/1","doc":"Attaches a Telemetry handler for Corsica events that logs through Logger . This function exists to mimic the behavior of the :log option that existed in Corsica v1.x. It attaches a handler for the following events: [:corsica, :accepted_request] [:corsica, :invalid_request] [:corsica, :rejected_request] The log levels can be customized through the :log_levels option that you can pass to this function. The levels and their defaults are: :accepted —  :debug :invalid —  :debug :rejected —  :warning The :log_levels option mirrors the :log option that you could pass to Corsica in v1.x. Usage We recommend calling this function in your application's Application.start/2 callback. Examples def start ( _type , _args ) do children = [ # ... ] Corsica.Telemetry . attach_default_handler ( log_levels : [ rejected : :error ] ) Supervisor . start_link ( children , strategy : :one_for_one ) end","ref":"Corsica.Telemetry.html#attach_default_handler/1"},{"type":"extras","title":"Common Issues","doc":"This page contains a list of common issues that can be encountered when setting up CORS and Corsica. These issues are not necessarily specific to Corsica but they bubble up in the Corsica issue tracker often enough to justify talking about them in the documentation.","ref":"common-issues.html"},{"type":"extras","title":"Common Issues - Browser gives a Origin ... is therefore not allowed access error","doc":"When attempting to perform a CORS request to your server, your browser might log the following error in the console: Response to preflight request doesn't pass access control check: No 'Access-Control-Allow-Origin' header is present on the requested resource. Origin 'example.com' is therefore not allowed access. This is a generic response that means your server is not allowing the CORS request. Some common reasons that can cause this are listed below. The Corsica plug is lower than any handling router in your plug pipeline (such as Plug.Router , a Phoenix.Router , or Plug.Static ). In order to be able to add CORS headers to responses and handle preflight requests, the Corsica plug (either with plug Corsica or as the router generated by use Corsica.Router ) must be higher in the plug pipeline than plugs that send a response down the connection. The server doesn't recognize the origin (the browser's current URL) as an allowed origin. Check the :origins option passed to Corsica. Make sure it allows whatever site the browser is trying perform the CORS request from. For example, if you are running a local JavaScript app on http://localhost:8100 , then the :origins option must contain http://localhost:8100 (or &quot;*&quot; to allow anything). Read the documentation for the Corsica module for more information on the possible values of the :origins option. The server doesn't allow the headers that the client is requesting. The client (such as the browser) can ask to use certain headers by sending a list of such headers in the Allow-Control-Request-Headers header included in the preflight request. For example, the client could ask to use Content-Type and Accept by passing Allow-Control-Request-Headers: Content-Type, Accept . The server (specifically, Corsica in this case) must respond to the preflight request and allow those headers by specifying them in the response Access-Control-Allow-Headers header. You can use the :allow_headers option passed to Corsica. In the example above, you would have to pass something like allow_headers: [&quot;content-type&quot;, &quot;accept&quot;] .","ref":"common-issues.html#browser-gives-a-origin-is-therefore-not-allowed-access-error"}]